# -*- coding: utf-8 -*-
"""
This is a collection of exceptions which are raised throughout the sourcecode.
"""

#external imports
#import ...
#internal imports
#import ..

class ImageBuildException(Exception):
  """
  This exception is raised any time an image fails to build and subuser is not at fault.
  """
  pass
