# -*- coding: utf-8 -*-

"""
This module defines collors for colloring text in an ANSI terminal.
The list is taken from http://stackoverflow.com/questions/287871/print-in-terminal-with-colors-using-python
"""

#external imports
#import ...
#internal imports
#import ...

HEADER = '\033[95m'
OKBLUE = '\033[94m'
OKGREEN = '\033[92m'
WARNING = '\033[93m'
FAIL = '\033[91m'
ENDC = '\033[0m'
BOLD = '\033[1m'
UNDERLINE = '\033[4m'
